from modernmetric.cls.base import MetricBase


class MetricBaseOperands(MetricBase):
    _needles = [
        "Token.Literal.Date",
        "Token.Literal.String.Double",
        "Token.Literal.String",
        "Token.Literal.Number.Bin",
        "Token.Literal.Number.Float",
        "Token.Literal.Number.Hex",
        "Token.Literal.Number.Integer.Long",
        "Token.Literal.Number.Integer",
        "Token.Literal.Number.Oct",
        "Token.Literal.Number",
        "Token.Name",
        "Token.Name.Attribute",
        "Token.Name.Builtin.Pseudo",
        "Token.Name.Builtin",
        "Token.Name.Constant",
        "Token.Name.Variable.Class",
        "Token.Name.Variable.Global",
        "Token.Name.Variable.Instance",
        "Token.Name.Variable.Magic",
        "Token.Name.Variable",
        "Token.Name.Other",
        "Token.Number.Bin",
        "Token.Number.Float",
        "Token.Number.Hex",
        "Token.Number.Integer.Long",
        "Token.Number.Integer",
        "Token.Number.Oct",
        "Token.Number",
        "Token.String.Char",
        "Token.String.Double",
        "Token.String.Escape",
        "Token.String.Heredoc",
        "Token.String.Interpol",
        "Token.String.Other",
        "Token.String.Regex",
        "Token.String.Single",
        "Token.String.Symbol"
    ]

    METRIC_OPERANDS_SUM = "operands_sum"
    METRIC_OPERANDS_UNIQUE = "operands_uniq"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self.__operands = []

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        for x in tokens:
            if str(x[0]) in MetricBaseOperands._needles:
                self.__operands.append(str(x[1]))

    def get_results(self):
        self._metrics[MetricBaseOperands.METRIC_OPERANDS_SUM] = len(self.__operands)
        self._metrics[MetricBaseOperands.METRIC_OPERANDS_UNIQUE] = len(list(set(self.__operands)))

        self._internalstore["operands"] = self.__operands
        return self._metrics

    def get_results_global(self, value_stores):
        _operands = []
        for x in self._get_all_matching_store_objects(value_stores):
            _operands += x["operands"]
        return {
            MetricBaseOperands.METRIC_OPERANDS_SUM: len(_operands),
            MetricBaseOperands.METRIC_OPERANDS_UNIQUE: len(list(set(_operands)))
        }



import sys
import chardet
from pygments import lexers
from pygments_tsx.tsx import patch_pygments

from modernmetric.cls.modules import get_modules_calculated
from modernmetric.cls.modules import get_modules_metrics
from modernmetric.cls.importer.filtered import FilteredImporter


patch_pygments()


def file_process(_file, _args, _importer):
    res = {}
    store = {}
    try:
        _lexer = lexers.get_lexer_for_filename(_file)
    except Exception as e:
        if _args.ignore_lexer_errors:
            # Printing to stderr since we write results to STDOUT
            print("Processing unknown file type: " + _file, file=sys.stderr)
            return (res, _file, "unknown", [], store)
        else:
            raise
    try:
        with open(_file, "rb") as i:
            _cnt = i.read()
            _enc = chardet.detect(_cnt)
            _cnt = _cnt.decode(_enc["encoding"]).encode("utf-8")
        _localImporter = {k: FilteredImporter(
            v, _file) for k, v in _importer.items()}
        tokens = list(_lexer.get_tokens(_cnt))
        if _args.dump:
            for x in tokens:
                print("{}: {} -> {}".format(_file, x[0], str(x[1])))
        else:
            _localMetrics = get_modules_metrics(_args, **_localImporter)
            _localCalc = get_modules_calculated(_args, **_localImporter)
            for x in _localMetrics:
                x.parse_tokens(_lexer.name, tokens)
                res.update(x.get_results())
                store.update(x.get_internal_store())
            for x in _localCalc:
                res.update(x.get_results(res))
                store.update(x.get_internal_store())
    except Exception:
        tokens = []
    return (res, _file, _lexer.name, tokens, store)



class MetricBase():
    def __init__(self, args, **kwargs):
        self._metrics = {"lang": []}
        self._internalstore = {}

    def parse_tokens(self, language, tokens):
        if language not in self._metrics["lang"]:
            self._metrics["lang"].append(language)

    def get_results(self):
        return self._metrics

    def get_internal_store(self):
        return {self.__class__.__name__: self._internalstore}

    def _get_all_matching_store_objects(self, store):
        res = []
        for item in store:
            if self.__class__.__name__ in item:
                res.append(item[self.__class__.__name__])
        return res

    def get_results_global(self, value_stores):
        return {}


class Importer():

    class ImporterItem():

        def __init__(self, _file, _cnt, _sev):
            self._values = {
                "filename": _file,
                "content": _cnt,
                "severity": _sev
            }

        def match(self, _filter):
            return all([self._values[k] == v for k, v in _filter.items()])

        def get(self):
            return self._values

        @staticmethod
        def from_csv(line):
            _sev = None
            if len(line) > 2:
                _sev = line[2]
            return Importer.ImporterItem(_file=line[0], _cnt=line[1], _sev=_sev)

    def __init__(self, args, filearg):
        self._input = filearg
        self._items = []

    def getItems(self, _filter={}):
        return [x.get() for x in self._items if x.match(_filter)]

    def getSumItems(self, _filter={}):
        _items = self.getItems(_filter=_filter)
        if len(_items) == 1:
            if str.isdigit(_items[0]["content"]):
                return int(_items[0]["content"])
        return len(_items)



import json
import sys

from modernmetric.cls.importer.base import Importer


class ImporterJSON(Importer):

    def __init__(self, args, filearg):
        super().__init__(args, filearg)
        self.__readInput()

    def __readInput(self):
        try:
            with open(self._input) as i:
                reader = json.load(i)
                for k, v in reader.items():
                    _sev = None
                    if "severity" in v:
                        _sev = v["severity"]
                    self._items.append(Importer.ImporterItem(_file=k, _cnt=v["content"], _sev=_sev))
        except Exception as e:
            sys.stderr.write("Read error: {}\n".format(e))



import math

from modernmetric.cls.base_calc import MetricBaseCalc
from modernmetric.cls.metric.cyclomatic import MetricBaseCyclomaticComplexity
from modernmetric.cls.metric.fanout import MetricBaseFanout
from modernmetric.cls.metric.loc import MetricBaseLOC

def fail_safe(fn):
    def wrapper(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except Exception as e:
            return 0
    return wrapper



class MetricBaseCalcTIOBE(MetricBaseCalc):

    METRIC_TIOBE_COVERAGE = "tiobe_coverage"
    METRIC_TIOBE_FUNCTIONAL = "tiobe_functional"
    METRIC_TIOBE_COMPLEXITY = "tiobe_complexity"
    METRIC_TIOBE_COMPILER = "tiobe_compiler"
    METRIC_TIOBE_STANDARD = "tiobe_standard"
    METRIC_TIOBE_DUPLICATION = "tiobe_duplication"
    METRIC_TIOBE_FANOUT = "tiobe_fanout"
    METRIC_TIOBE_SECURITY = "tiobe_security"
    METRIC_TIOBE = "tiobe"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self.__addArgs = kwargs
    
    @fail_safe
    def __getScaledValue(self, metrics, value):
        return 100.0 / ((value) / (metrics[MetricBaseLOC.METRIC_LOC] / 1000.0) + 1.0)

    @fail_safe
    def __getFromImporter(self, section, _default=0.0):
        if "import_{}".format(section) in self.__addArgs:
            return self.__addArgs["import_{}".format(section)].getSumItems()
        return _default

    @fail_safe
    def __getTiobeComplexity(self, metrics):
        cc = metrics[MetricBaseCyclomaticComplexity.METRIC_CYCLOMATIC_COMPLEXITY]
        return 6400.0 / float(math.pow(cc, 3) - math.pow(cc, 2) - cc + 65)

    @fail_safe
    def __getTiobeFanout(self, metrics):
        _int = metrics[MetricBaseFanout.METRIC_FANOUT_INTERNAL]
        _ext = metrics[MetricBaseFanout.METRIC_FANOUT_EXTERNAL]
        return float(min(max(120.0 - ((8.0 * _int) + (2.0 * _ext)), 0.0), 100.0))

    @fail_safe
    def __getTiobeCoverage(self, metrics):
        _per = self.__getFromImporter("coverage", _default=100.0)
        if _per < 1.0:
            # No coverage
            return 0.0
        return min(((0.75 * _per) + 32.5), 100.0)

    @fail_safe
    def __getTiobeStandard(self, metrics):
        return self.__getScaledValue(metrics, self.__getFromImporter("standard"))

    @fail_safe
    def __getTiobeSecurity(self, metrics):
        return self.__getScaledValue(metrics, self.__getFromImporter("security"))

    @fail_safe
    def __getTiobeDuplication(self, metrics):
        return min(-30.0 * math.log10(self.__getFromImporter("duplication") or 0.00001) +
                   70.0, 100.0)
    
    @fail_safe
    def __getTiobeCompiler(self, metrics):
        _violations = self.__getScaledValue(metrics, self.__getFromImporter("compiler"))
        return max(100.0 - 50.0 *
                   math.log((101 - _violations) or 0.00001), 0.0)

    @fail_safe
    def __getTiobeFunctional(self, metrics):
        _violations = self.__getFromImporter("functional")
        return max(self.__getScaledValue(metrics, _violations) * 2.0 - 100.0, 0.0)

    @fail_safe
    def __getTiobe(self, metrics):
        return (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COVERAGE] * 0.2) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_FUNCTIONAL] * 0.2) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COMPLEXITY] * 0.15) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COMPILER] * 0.15) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_STANDARD] * 0.1) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_DUPLICATION] * 0.1) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_FANOUT] * 0.05) + \
               (metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_SECURITY] * 0.05)

    def get_results(self, metrics):
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COMPLEXITY] = self.__getTiobeComplexity(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_FANOUT] = self.__getTiobeFanout(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COMPILER] = self.__getTiobeCompiler(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_COVERAGE] = self.__getTiobeCoverage(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_DUPLICATION] = self.__getTiobeDuplication(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_FUNCTIONAL] = self.__getTiobeFunctional(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_SECURITY] = self.__getTiobeSecurity(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE_STANDARD] = self.__getTiobeStandard(metrics)
        metrics[MetricBaseCalcTIOBE.METRIC_TIOBE] = self.__getTiobe(metrics)
        return super().get_results(metrics)



from modernmetric.cls.base import MetricBase


class MetricBaseCyclomaticComplexity(MetricBase):

    __exitPoints = [
        "return",
        "exit",
        "assert",
        "break",
        "continue",
        "yield"
    ]

    __conditions = [
        "if",
        "else",
        "elif",
        "case",
        "default",
        "for",
        "while",
        "and",
        "or",
        "&&",
        "||"
    ]

    METRIC_CYCLOMATIC_COMPLEXITY = "cyclomatic_complexity"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self._internalstore["exitpoints"] = 0
        self._internalstore["conditions"] = 0

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        for x in tokens:
            if str(x[1]) in MetricBaseCyclomaticComplexity.__exitPoints:
                self._internalstore["exitpoints"] += 1
            if str(x[1]) in MetricBaseCyclomaticComplexity.__conditions:
                self._internalstore["conditions"] += 1

    def get_results(self):
        self._metrics[MetricBaseCyclomaticComplexity.METRIC_CYCLOMATIC_COMPLEXITY] = max(
            self._internalstore["conditions"] - self._internalstore["exitpoints"] + 2, 0)
        return self._metrics

    def get_results_global(self, value_stores):
        __exitPoints = 0
        __conditions = 0
        for x in self._get_all_matching_store_objects(value_stores):
            __conditions += x["conditions"]
            __exitPoints += x["exitpoints"]
        return {
            MetricBaseCyclomaticComplexity.METRIC_CYCLOMATIC_COMPLEXITY: max(
                __conditions - __exitPoints + 2, 0)
        }



from modernmetric.cls.base import MetricBase


class MetricBaseComments(MetricBase):
    _needles = [
        "Token.Comment",
        "Token.Comment.Hashbang",
        "Token.Comment.Multiline",
        "Token.Comment.Single",
        "Token.Comment.Special",
        "Token.Literal.String.Doc"
    ]

    _specific = {
        "Python": [
            "Token.Comment.Preproc"
        ]
    }

    METRIC_COMMENT_RATIO = "comment_ratio"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self.__overall = 0
        self.__comments = 0

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        _n = MetricBaseComments._needles
        if language in MetricBaseComments._specific:
            _n += MetricBaseComments._specific[language]
        for x in tokens:
            self.__overall += len(str(x[1]))
            if str(x[0]) in _n:
                self.__comments += len(str(x[1]))

    def get_results(self):
        if self.__overall == 0:
            # sanity
            self.__overall = 1
        self._metrics[MetricBaseComments.METRIC_COMMENT_RATIO] = self.__comments * \
            100.0 / float(self.__overall)
        self._internalstore["comments"] = self.__comments
        self._internalstore["overall"] = self.__overall
        return self._metrics

    def get_results_global(self, value_stores):
        __comments = 0
        __overall = 0
        for x in self._get_all_matching_store_objects(value_stores):
            __comments += x["comments"]
            __overall += x["overall"]
        return {
            MetricBaseComments.METRIC_COMMENT_RATIO: __comments * 100.0 / float(__overall or 1.0)
        }



from modernmetric.cls.base_calc import MetricBaseCalc
from modernmetric.cls.metric.operators import MetricBaseOperator
from modernmetric.cls.metric.operands import MetricBaseOperands


class MetricBaseCalcPylint(MetricBaseCalc):

    METRIC_PYLINT = "pylint"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self.__addArgs = kwargs

    def __getFromImporter(self, section, severity, _default=0.0):
        if "import_{}".format(section) in self.__addArgs:
            return self.__addArgs["import_{}".format(section)].getSumItems(
                _filter={"severity": severity}
            )
        return _default

    def __getError(self, metrics):
        return (self.__getFromImporter("compiler", "error") +
                self.__getFromImporter("functional", "error") +
                self.__getFromImporter("standard", "error") +
                self.__getFromImporter("security", "error")) * 5.0

    def __getWarning(self, metrics):
        return self.__getFromImporter("compiler", "warning") + \
            self.__getFromImporter("functional", "warning") + \
            self.__getFromImporter("standard", "warning") + \
            self.__getFromImporter("security", "warning")

    def __getInfo(self, metrics):
        return self.__getFromImporter("compiler", "info") + \
            self.__getFromImporter("functional", "info") + \
            self.__getFromImporter("standard", "info") + \
            self.__getFromImporter("security", "info")

    def __getScore(self, metrics):
        # 100.0 - ((float(5 * error + warning + refactor + convention) / statement) * 100.0)
        _statements = (metrics[MetricBaseOperands.METRIC_OPERANDS_SUM] +
                       metrics[MetricBaseOperator.METRIC_OPERATORS_SUM]) or 1.0
        return max(100.0 - ((float(self.__getError(metrics) +
                                   self.__getWarning(metrics) +
                                   self.__getInfo(metrics)) /
                            _statements) * 100.0), 0.0)

    def get_results(self, metrics):
        metrics[MetricBaseCalcPylint.METRIC_PYLINT] = self.__getScore(metrics)
        return super().get_results(metrics)



from modernmetric.cls.base import MetricBase


class MetricBaseFanout(MetricBase):
    _needles = {
        "Python": [
            "Token.Name.Namespace"
        ],
        "C": [
            "Token.Comment.PreprocFile"
        ],
        "C++": [
            "Token.Comment.PreprocFile"
        ]
    }
    _functions = {
        "PHP": "_parsePHP",
        "Go": "_parseGo",
        "Ruby": "_parseRuby"
    }
    _internal = {
        "Python": {"start": ".", "end": ""},
        "C": {"start": "\"", "end": "\""},
        "C++": {"start": "\"", "end": "\""}
    }

    METRIC_FANOUT_INTERNAL = "fanout_internal"
    METRIC_FANOUT_EXTERNAL = "fanout_external"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self._int = set()
        self._ext = set()

    def __isInternal(self, value, internal_mapping):
        return all([value.startswith(internal_mapping["start"]),
                    value.endswith(internal_mapping["end"])])

    def _parsePHP(self, iterator):
        res = []
        _start_token = ["include", "require", "include_once", "require_once"]
        _cont_token = ["Token.Literal.String.Single", "Token.Literal.String.Double"]
        for i, val in iterator:
            if str(val[0]) in ["Token.Keyword"] and \
               val[1] in _start_token:
                while iterator and \
                      str(val[0]) not in _cont_token:
                    i, val = next(iterator)
                if iterator:
                    res.append(val[1].strip("'").strip('"'))
        return res

    def _parseRuby(self, iterator):
        res = []
        for i, val in iterator:
            if str(val[0]) in ["Token.Name.Builtin"] and val[1] in ["require"]:
                while iterator:
                    i, val = next(iterator)
                    if str(val[0]) in ["Token.Literal.String.Single"]:
                        print()
                        res.append(val[1].strip("'").strip('"'))
                        break
                    elif str(val[0]) in ["Token.Text"] and val[1] in ["\n", "\r\n"]:
                        break
        return res

    def _parseGo(self, iterator):
        res = []
        for i, val in iterator:
            if str(val[0]) in ["Token.Keyword.Namespace"] and val[1] in ["import"]:
                while iterator:
                    i, val = next(iterator)
                    if str(val[0]) in ["Token.Literal.String"]:
                        res.append(val[1].strip("'").strip('"'))
                    if str(val[0]) in ["Token.Punctuation"] and val[1] in [')']:
                        break
        return res

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        if language in MetricBaseFanout._internal:
            _i = MetricBaseFanout._internal[language]
        else:
            _i = {"start": "", "end": ""}
        _imports = []
        if language in MetricBaseFanout._needles.keys():
            _n = MetricBaseFanout._needles[language]
            for x in [x for x in tokens if str(x[0]) in _n]:
                _imports.append(x[1])
        elif language in MetricBaseFanout._functions:
            _imports = getattr(self,
                               MetricBaseFanout._functions[language])(enumerate(tokens))
        # else:
        #     # Language isn't supported at the moment
        #     for x in tokens:
        #         print(str(x))
        for x in _imports:
            if self.__isInternal(x, _i):
                self._int.add(str(x))
            else:
                self._ext.add(str(x))
        self._metrics.update({MetricBaseFanout.METRIC_FANOUT_INTERNAL: len(list(self._int)),
                              MetricBaseFanout.METRIC_FANOUT_EXTERNAL: len(list(self._ext))})
        self._internalstore["int"] = list(self._int)
        self._internalstore["ext"] = list(self._ext)

    def get_results_global(self, value_stores):
        _int = []
        _ext = []
        for x in self._get_all_matching_store_objects(value_stores):
            _int += x["int"]
            _ext += x["ext"]
        return {
            MetricBaseFanout.METRIC_FANOUT_INTERNAL: len(_int),
            MetricBaseFanout.METRIC_FANOUT_EXTERNAL: len(_ext)
        }



import statistics

from modernmetric.cls.base_stats import MetricBaseStats


class MetricBaseStatsAverage(MetricBaseStats):

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)

    def _getInputList(self, metrics, key):
        res = []
        for _, v in metrics.items():
            if key in v and not isinstance(v[key], list):
                res.append(v[key])
        return res

    def get_results(self, metrics, files="files", overall="overall"):
        _keylist = list(metrics[overall].keys())
        metrics["stats"] = {}
        metrics["stats"]["mean"] = {}
        metrics["stats"]["max"] = {}
        metrics["stats"]["min"] = {}
        if len(metrics["files"]) > 1:
            metrics["stats"]["sd"] = {}
        metrics["stats"]["median"] = {}
        for k in _keylist:
            _list = self._getInputList(metrics[files], k)
            if not _list:
                continue
            metrics["stats"]["mean"][k] = statistics.mean(_list)
            metrics["stats"]["median"][k] = statistics.median(_list)
            metrics["stats"]["max"][k] = max(_list)
            metrics["stats"]["min"][k] = min(_list)
            if len(metrics["files"]) > 1:
                metrics["stats"]["sd"][k] = statistics.stdev(_list)
        return super().get_results(metrics, files="files", overall="overall")



from modernmetric.cls.base import MetricBase
import pygments
import pygments.lexer
import pygments.lexers
import pygments.token
import pygments.util

from pygount.analysis import (
    white_characters, 
    white_code_words, 
    _delined_tokens,
    _pythonized_comments
)


def _line_parts(tokens, language_id: str):
    line_marks = set()
    white_text = " \f\n\r\t" + white_characters(language_id)
    white_words = white_code_words(language_id)
    for token_type, token_text in tokens:
        # NOTE: Pygments treats preprocessor statements as special comments.
        is_actual_comment = token_type in pygments.token.Comment and token_type not in (
            pygments.token.Comment.Preproc,
            pygments.token.Comment.PreprocFile,
        )
        if is_actual_comment:
            line_marks.add("d")  # 'documentation'
        elif token_type in pygments.token.String:
            line_marks.add("s")  # 'string'
        else:
            is_white_text = (token_text.strip() in white_words) or (token_text.rstrip(white_text) == "")
            if not is_white_text:
                line_marks.add("c")  # 'code'
            
        if token_text.endswith("\n"):
            yield line_marks
            line_marks = set()
    if len(line_marks) >= 1:
        yield line_marks


class MetricBaseLOC(MetricBase):
    METRIC_LOC = "loc"
    METRIC_CLOC = "code_loc"
    METRIC_DLOC = "documentation_loc"
    METRIC_SLOC = "string_loc"
    METRIC_ELOC = "empty_loc"

    metrics = {
        "c": METRIC_CLOC,
        "d": METRIC_DLOC,
        "e": METRIC_ELOC,
        "s": METRIC_SLOC
    }

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        tokens = _delined_tokens(tokens)
        if language.lower() == "python":
            tokens = _pythonized_comments(tokens)
        language_id = language.lower()
        mark_to_count_map = {"c": 0, "d": 0, "e": 0, "s": 0}
        for line_parts in _line_parts(tokens, language_id):
            mark_to_increment = "e"
            for mark_to_check in ("d", "s", "c"):
                if mark_to_check in line_parts:
                    mark_to_increment = mark_to_check
            mark_to_count_map[mark_to_increment] += 1

        self._metrics[MetricBaseLOC.METRIC_LOC] = 0
        for mark_type in mark_to_count_map:
            if MetricBaseLOC.metrics[mark_type] not in self._metrics:
                self._metrics[MetricBaseLOC.metrics[mark_type]] = 0    
            self._metrics[MetricBaseLOC.metrics[mark_type]] += mark_to_count_map[mark_type]
            if mark_type == 'c' or mark_type != 's':
                self._metrics[MetricBaseLOC.METRIC_LOC] += mark_to_count_map[mark_type]
            self._metrics[MetricBaseLOC.metrics[mark_type]] = max(mark_to_count_map[mark_type],1)
        self._metrics[MetricBaseLOC.METRIC_LOC] = max(self._metrics[MetricBaseLOC.METRIC_LOC], 1)
        self._metrics[MetricBaseLOC.METRIC_CLOC] = max(self._metrics[MetricBaseLOC.METRIC_CLOC], 1)
        self._metrics[MetricBaseLOC.METRIC_DLOC] = max(self._metrics[MetricBaseLOC.METRIC_DLOC], 1)
        self._metrics[MetricBaseLOC.METRIC_ELOC] = max(self._metrics[MetricBaseLOC.METRIC_ELOC], 1)
        self._metrics[MetricBaseLOC.METRIC_SLOC] = max(self._metrics[MetricBaseLOC.METRIC_SLOC], 1)
        
        self._internalstore[MetricBaseLOC.METRIC_LOC] = self._metrics[MetricBaseLOC.METRIC_LOC]
        self._internalstore[MetricBaseLOC.METRIC_CLOC] = self._metrics[MetricBaseLOC.METRIC_CLOC]
        self._internalstore[MetricBaseLOC.METRIC_DLOC] = self._metrics[MetricBaseLOC.METRIC_DLOC]
        self._internalstore[MetricBaseLOC.METRIC_ELOC] = self._metrics[MetricBaseLOC.METRIC_ELOC]
        self._internalstore[MetricBaseLOC.METRIC_SLOC] = self._metrics[MetricBaseLOC.METRIC_SLOC]

    def get_results_global(self, value_stores):
        _sum_LOC = sum([x[MetricBaseLOC.METRIC_LOC] for x in self._get_all_matching_store_objects(value_stores)])
        _sum_CLOC = sum([x[MetricBaseLOC.METRIC_CLOC] for x in self._get_all_matching_store_objects(value_stores)])
        _sum_DLOC = sum([x[MetricBaseLOC.METRIC_DLOC] for x in self._get_all_matching_store_objects(value_stores)])
        _sum_ELOC = sum([x[MetricBaseLOC.METRIC_ELOC] for x in self._get_all_matching_store_objects(value_stores)])
        _sum_SLOC = sum([x[MetricBaseLOC.METRIC_SLOC] for x in self._get_all_matching_store_objects(value_stores)])
        return { 
            MetricBaseLOC.METRIC_LOC: _sum_LOC,
            MetricBaseLOC.METRIC_CLOC: _sum_CLOC,
            MetricBaseLOC.METRIC_DLOC: _sum_DLOC,
            MetricBaseLOC.METRIC_ELOC: _sum_ELOC,
            MetricBaseLOC.METRIC_SLOC: _sum_SLOC
        }


class FilteredImporter():
    def __init__(self, importer, filename):
        self.__importer = importer
        self.__filefilter = filename

    def getItems(self):
        return self.__importer.getItems({"filename": self.__filefilter})

    def getSumItems(self, _filter={}):
        return self.__importer.getSumItems(_filter={"filename": self.__filefilter, **_filter})



import subprocess
import setuptools

import os
import datetime


# Get the version from the git tag, and write to VERSION.
ref = None
if "GITHUB_REF" in os.environ:
    ref = os.environ["GITHUB_REF"]

if ref and ref is not None and ref.startswith("refs/tags/"):
    version = ref.replace("refs/tags/", "")
else:
    version = datetime.datetime.now().strftime("%Y.%m.%d%H%M%S")

print(version)

_long_description = "See https://github.com/startupos/modernmetric for documentation"  # noqa:E501
_long_description_content_type = "text/plain"
try:
    _long_description = subprocess.check_output(
        ["pandoc", "--from", "markdown", "--to", "rst", "README.md"]
    ).decode("utf-8")
    _long_description_content_type = "text/x-rst"
except (subprocess.CalledProcessError, FileNotFoundError):
    pass

requirements = []
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name="modernmetric",
    version=version,
    author="Jason Nichols",
    author_email="jason@startupos.dev",
    description="Calculate code metrics in various languages",
    long_description=_long_description,
    long_description_content_type=_long_description_content_type,
    url="https://github.com/startupos/modernmetric",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "modernmetric = modernmetric.__main__:main",
        ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Quality Assurance",
    ],
)



import os
import sys

from modernmetric.cls.importer.base import Importer
from modernmetric.cls.importer.mods.csv import ImporterCSV
from modernmetric.cls.importer.mods.json import ImporterJSON


def importer_pick(args, filearg):
    if not filearg:
        return None
    _file, _ext = os.path.splitext(filearg)
    if _ext in [".csv"]:
        return ImporterCSV(args, filearg)
    if _ext in [".json"]:
        return ImporterJSON(args, filearg)
    sys.stderr.write("No supported importer found for {}\n".format(_ext))
    return Importer(args, None)



from modernmetric.cls.calc.halstead import MetricBaseCalcHalstead
from modernmetric.cls.calc.maintenance import MetricBaseCalcMaintenanceIndex
from modernmetric.cls.calc.pylint import MetricBaseCalcPylint
from modernmetric.cls.calc.tiobe import MetricBaseCalcTIOBE
from modernmetric.cls.metric.comments import MetricBaseComments
from modernmetric.cls.metric.cyclomatic import MetricBaseCyclomaticComplexity
from modernmetric.cls.metric.fanout import MetricBaseFanout
from modernmetric.cls.metric.loc import MetricBaseLOC
from modernmetric.cls.metric.operands import MetricBaseOperands
from modernmetric.cls.metric.operators import MetricBaseOperator
from modernmetric.cls.stats.stats import MetricBaseStatsAverage


def get_modules_metrics(args, **kwargs):
    return [
        MetricBaseComments(args, **kwargs),
        MetricBaseCyclomaticComplexity(args, **kwargs),
        MetricBaseFanout(args, **kwargs),
        MetricBaseLOC(args, **kwargs),
        MetricBaseOperands(args, **kwargs),
        MetricBaseOperator(args, **kwargs),
    ]


def get_modules_calculated(args, **kwargs):
    return [
        MetricBaseCalcHalstead(args, **kwargs),
        MetricBaseCalcMaintenanceIndex(args, **kwargs),
        MetricBaseCalcTIOBE(args, **kwargs),
        MetricBaseCalcPylint(args, **kwargs)
    ]


def get_modules_stats(args, **kwargs):
    return [
        MetricBaseStatsAverage(args, **kwargs)
    ]


def get_additional_parser_args(parser):
    parser.add_argument("--bugpredict",
                        choices=MetricBaseCalcHalstead.BUGPRED_METHOD.keys(),
                        default=MetricBaseCalcHalstead.BUGPRED_DEFAULT,
                        help="Method how to calculate the bug prediction",
                        dest="halstead_bug_predict_method")
    parser.add_argument("--maintindex",
                        choices=MetricBaseCalcMaintenanceIndex.MI_METHOD.keys(),
                        default=MetricBaseCalcMaintenanceIndex.MI_DEFAULT,
                        help="Method how to calculate the maintainability index",
                        dest="maintenance_index_calc_method")
